@@ -42,6 +42,7 @@
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
+#include <gtk/gtkx.h>
 #include <gdk/gdkx.h>
 
 #include <glib.h>
@@ -441,9 +442,9 @@ setup_special_icon_list(GtkBuilder *gxml,
         GdkPixbuf *pix = NULL;
 
         if(gtk_icon_theme_has_icon(itheme, icons[i].icon))
-            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon, w, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon, w, 0, NULL);
         else
-            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon_fallback, w, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+            pix = gtk_icon_theme_load_icon(itheme, icons[i].icon_fallback, w, 0, NULL);
 
         if(i < REMOVABLE_DEVICES) {
             gtk_tree_store_append(ts, &parent_iter, NULL);
@@ -469,7 +470,6 @@ setup_special_icon_list(GtkBuilder *gxml,
     treeview = GTK_WIDGET(gtk_builder_get_object(gxml, "treeview_default_icons"));
     g_object_set_data(G_OBJECT(treeview), "xfconf-channel", channel);
     col = gtk_tree_view_column_new();
-    gtk_tree_view_column_set_spacing(col, 6);
     gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
 
     render = gtk_cell_renderer_toggle_new();
@@ -721,7 +721,11 @@ xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
 
     workspace_name = g_strdup(wnck_workspace_get_name(workspace));
 
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    if(gdk_display_get_n_monitors(gtk_widget_get_display(panel->chk_apply_to_all)) > 1) {
+#else
     if(gdk_screen_get_n_monitors(gtk_widget_get_screen(panel->chk_apply_to_all)) > 1) {
+#endif
         if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(panel->chk_apply_to_all))) {
             /* Multi-monitor single workspace */
             if(panel->monitor_name) {
@@ -1336,72 +1340,76 @@ xfdesktop_settings_background_tab_change_bindings(AppearancePanel *panel,
     g_free(buf);
 
     /* color 1 button */
-    buf = xfdesktop_settings_generate_per_workspace_binding_string(panel, "color1");
+    /* Fixme: we will need to migrate from color1 to rgba1 (GdkColor to GdkRGBA) */
+    buf = xfdesktop_settings_generate_per_workspace_binding_string(panel, "rgba1");
     if(remove_binding) {
         xfconf_g_property_unbind(panel->color1_btn_id);
     } else {
         /* If the first color doesn't exist, try to load the old one */
         if(!xfconf_channel_has_property(channel, buf)) {
             GValue value = { 0, };
-            old_property = xfdesktop_settings_generate_old_binding_string(panel, "color1");
+            old_property = xfdesktop_settings_generate_old_binding_string(panel, "rgba1");
 
             xfconf_channel_get_property(channel, old_property, &value);
 
             if(G_VALUE_HOLDS_BOXED(&value)) {
-                gtk_color_button_set_color(GTK_COLOR_BUTTON(panel->color1_btn),
+                gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(panel->color1_btn),
                                            g_value_get_boxed(&value));
                 g_value_unset(&value);
             } else {
                 /* revert to showing our default color */
-                GdkColor color1;
-                color1.red = 0x1515;
-                color1.green = 0x2222;
-                color1.blue = 0x3333;
-                gtk_color_button_set_color(GTK_COLOR_BUTTON(panel->color1_btn), &color1);
+                GdkRGBA color1;
+                color1.red = 0.5f;
+                color1.green = 0.5f;
+                color1.blue = 0.5f;
+                color1.alpha = 1.0f;
+                gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(panel->color1_btn), &color1);
             }
 
             g_free(old_property);
         }
 
         /* Now bind to the new value */
-        panel->color1_btn_id = xfconf_g_property_bind_gdkcolor(channel, buf,
-                                                               G_OBJECT(panel->color1_btn),
-                                                               "color");
+        panel->color1_btn_id = xfconf_g_property_bind_gdkrgba(channel, buf,
+                                                              G_OBJECT(panel->color1_btn),
+                                                              "rgba");
     }
     g_free(buf);
 
     /* color 2 button */
-    buf = xfdesktop_settings_generate_per_workspace_binding_string(panel, "color2");
+    /* Fixme: we will need to migrate from color1 to rgba1 (GdkColor to GdkRGBA) */
+    buf = xfdesktop_settings_generate_per_workspace_binding_string(panel, "rgba2");
     if(remove_binding) {
         xfconf_g_property_unbind(panel->color2_btn_id);
     } else {
         /* If the 2nd color doesn't exist, try to load the old one */
         if(!xfconf_channel_has_property(channel, buf)) {
             GValue value = { 0, };
-            old_property = xfdesktop_settings_generate_old_binding_string(panel, "color2");
+            old_property = xfdesktop_settings_generate_old_binding_string(panel, "rgba2");
 
             xfconf_channel_get_property(channel, old_property, &value);
 
             if(G_VALUE_HOLDS_BOXED(&value)) {
-                gtk_color_button_set_color(GTK_COLOR_BUTTON(panel->color2_btn),
+                gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(panel->color2_btn),
                                            g_value_get_boxed(&value));
                 g_value_unset(&value);
             } else {
                 /* revert to showing our default color */
-                GdkColor color2;
-                color2.red = 0x1515;
-                color2.green = 0x2222;
-                color2.blue = 0x3333;
-                gtk_color_button_set_color(GTK_COLOR_BUTTON(panel->color2_btn), &color2);
+                GdkRGBA color2;
+                color2.red = 0.5f;
+                color2.green = 0.5f;
+                color2.blue = 0.5f;
+                color2.alpha = 1.0f;
+                gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(panel->color2_btn), &color2);
             }
 
             g_free(old_property);
         }
 
         /* Now bind to the new value */
-        panel->color2_btn_id = xfconf_g_property_bind_gdkcolor(channel, buf,
-                                                               G_OBJECT(panel->color2_btn),
-                                                               "color");
+        panel->color2_btn_id = xfconf_g_property_bind_gdkrgba(channel, buf,
+                                                              G_OBJECT(panel->color2_btn),
+                                                              "rgba");
     }
     g_free(buf);
 
@@ -1475,6 +1483,25 @@ suboptions_set_sensitive(GtkToggleButton *btn,
     gtk_widget_set_sensitive(box, gtk_toggle_button_get_active(btn));
 }
 
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+static gint
+display_get_monitor_num(GdkDisplay *display, GdkMonitor *monitor)
+{
+    gint i;
+
+    g_return_val_if_fail(GDK_IS_DISPLAY(display), 0);
+    g_return_val_if_fail(GDK_IS_MONITOR(monitor), 0);
+
+    for(i=0; i<gdk_display_get_n_monitors(display); i++) {
+        if(monitor == gdk_display_get_monitor(display, i))
+            return i;
+    }
+
+    g_warning("unable to get the monitor number");
+    return 0;
+}
+#endif
+
 static void
 cb_update_background_tab(WnckWindow *wnck_window,
                          gpointer user_data)
@@ -1483,21 +1510,32 @@ cb_update_background_tab(WnckWindow *wnck_window,
     gint screen_num, monitor_num, workspace_num;
     gchar *monitor_name = NULL;
     WnckWorkspace *wnck_workspace = NULL;
-    GdkScreen *screen;
+    WnckScreen    *wnck_screen = NULL;
+    GdkWindow     *window = NULL;
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    GdkDisplay    *display = NULL;
+    GdkMonitor    *monitor = NULL;
+#endif
 
     /* If we haven't found our window return now and wait for that */
     if(panel->wnck_window == NULL)
         return;
 
     /* Get all the new settings for comparison */
-    screen = gtk_widget_get_screen(panel->image_iconview);
-    wnck_workspace = wnck_window_get_workspace(wnck_window);
+    wnck_screen = wnck_window_get_screen(panel->wnck_window);
+    wnck_workspace = wnck_window_get_workspace(panel->wnck_window);
 
     workspace_num = xfdesktop_settings_get_active_workspace(panel, wnck_window);
-    screen_num = gdk_screen_get_number(screen);
-    monitor_num = gdk_screen_get_monitor_at_window(screen,
-                                                   gtk_widget_get_window(panel->image_iconview));
-    monitor_name = gdk_screen_get_monitor_plug_name(screen, monitor_num);
+    screen_num = wnck_screen_get_number(wnck_screen);
+    window = gtk_widget_get_window(panel->image_iconview);
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    display = gdk_window_get_display(window);
+    monitor = gdk_display_get_monitor_at_window(display, window);
+    monitor_num = display_get_monitor_num(display, monitor);
+#else
+    monitor_num = gdk_screen_get_monitor_at_window(gtk_widget_get_screen(panel->image_iconview), window);
+#endif
+    monitor_name = xfdesktop_get_monitor_name_from_gtk_widget(panel->image_iconview, monitor_num);
 
     /* Most of the time we won't change monitor, screen, or workspace so try
      * to bail out now if we can */
@@ -1531,7 +1569,7 @@ cb_update_background_tab(WnckWindow *wnck_window,
     panel->workspace = workspace_num;
     panel->screen = screen_num;
     panel->monitor = monitor_num;
-    panel->monitor_name = gdk_screen_get_monitor_plug_name(screen, panel->monitor);
+    panel->monitor_name = xfdesktop_get_monitor_name_from_gtk_widget(panel->image_iconview, monitor_num);
 
     /* The first monitor has the option of doing the "spanning screens" style,
      * but only if there's multiple monitors attached. Remove it in all other cases.
@@ -1540,10 +1578,17 @@ cb_update_background_tab(WnckWindow *wnck_window,
      */
     gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(panel->image_style_combo),
                               XFCE_BACKDROP_IMAGE_SPANNING_SCREENS);
-    if(panel->monitor == 0 && gdk_screen_get_n_monitors(screen) > 1) {
+#if 0 /* GTK_CHECK_VERSION (3, 22, 0) */
+    if(panel->monitor == 0 && gdk_display_get_n_monitors(gtk_widget_get_display(panel->image_style_combo)) > 1) {
         gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(panel->image_style_combo),
                                        _("Spanning screens"));
     }
+#else
+    if(panel->monitor == 0 && gdk_screen_get_n_monitors(gtk_widget_get_screen(panel->image_iconview)) > 1) {
+        gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(panel->image_style_combo),
+                                       _("Spanning screens"));
+    }
+#endif
 
     /* connect the new bindings */
     xfdesktop_settings_background_tab_change_bindings(panel,
@@ -1718,7 +1763,6 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     GdkScreen *screen;
     WnckScreen *wnck_screen;
     XfconfChannel *channel = panel->channel;
-    GdkColor color;
     const gchar *path;
     GFile *file;
     gchar *uri_path;
@@ -1779,7 +1823,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     chk_show_thumbnails = GTK_WIDGET(gtk_builder_get_object(main_gxml,
                                                             "chk_show_thumbnails"));
     /* The default value when this property is not set, is 'TRUE'.
-     * the bind operation defaults to 'FALSE' for unset boolean properties. 
+     * the bind operation defaults to 'FALSE' for unset boolean properties.
      *
      * Make the checkbox correspond to the default behaviour.
      */
@@ -1813,7 +1857,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     {
         g_printerr("Failed to parse appearance settings UI description: %s\n",
                    error->message);
-        g_error_free(error);
+        g_clear_error(&error);
         exit(1);
     }
 
@@ -1831,12 +1875,8 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                                                        "infobar_header"));
 
     panel->infobar_label = gtk_label_new("This is some text");
-    gdk_color_parse ("black", &color);
-    gtk_widget_modify_fg (panel->infobar_label, GTK_STATE_NORMAL, &color);
-    gtk_widget_set_no_show_all(panel->infobar, TRUE);
     gtk_widget_show(panel->infobar_label);
     gtk_widget_show(panel->infobar);
-    gtk_widget_set_visible(panel->infobar, FALSE);
 
     /* Add the panel's infobar label to the infobar, with this setup
      * it's easy to update the text for the infobar. */
@@ -1863,6 +1903,9 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     gtk_file_filter_add_mime_type(filter, "text/directory");
     gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(panel->btn_folder), filter);
 
+    /* Change the title of the file chooser dialog */
+    gtk_file_chooser_button_set_title(GTK_FILE_CHOOSER_BUTTON(panel->btn_folder), _("Select a Directory"));
+
     /* Get default wallpaper folder */
     path = system_data_lookup ();
 
@@ -1871,7 +1914,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
         uri_path = g_file_get_uri (file);
 
         gtk_file_chooser_add_shortcut_folder_uri (GTK_FILE_CHOOSER(panel->btn_folder),
-                                                  uri_path, NULL); 
+                                                  uri_path, NULL);
 
         g_free (uri_path);
         g_object_unref (file);
@@ -2049,7 +2092,7 @@ xfdesktop_settings_response(GtkWidget *dialog, gint response_id, gpointer user_d
     }
 }
 
-static GdkNativeWindow opt_socket_id = 0;
+static Window opt_socket_id = 0;
 static gboolean opt_version = FALSE;
 static gboolean opt_enable_debug = FALSE;
 static GOptionEntry option_entries[] = {
@@ -2085,7 +2128,7 @@ main(int argc, char **argv)
             g_printerr("%s: %s.\n", G_LOG_DOMAIN, error->message);
             g_printerr(_("Type '%s --help' for usage."), G_LOG_DOMAIN);
             g_printerr("\n");
-            g_error_free(error);
+            g_clear_error(&error);
         } else
             g_error("Unable to open display.");
 
@@ -2104,12 +2147,12 @@ main(int argc, char **argv)
 
     if(!xfconf_init(&error)) {
         xfce_message_dialog(NULL, _("Desktop Settings"),
-                            GTK_STOCK_DIALOG_ERROR,
+                            "dialog-error",
                             _("Unable to contact settings server"),
                             error->message,
-                            GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT,
+                            XFCE_BUTTON_TYPE_MIXED, "application-exit", _("Quit"), GTK_RESPONSE_ACCEPT,
                             NULL);
-        g_error_free(error);
+        g_clear_error(&error);
         return 1;
     }
 
@@ -2120,7 +2163,7 @@ main(int argc, char **argv)
                                     &error))
     {
         g_printerr("Failed to parse UI description: %s\n", error->message);
-        g_error_free(error);
+        g_clear_error(&error);
         return 1;
     }
 
@@ -2141,7 +2184,7 @@ main(int argc, char **argv)
              xfconf_channel_get_int(channel, SETTINGS_WINDOW_LAST_HEIGHT, -1));
         gtk_window_present(GTK_WINDOW (dialog));
 
-        screen = gdk_screen_get_number(gtk_widget_get_screen(dialog));
+        screen = XScreenNumberOfScreen(gdk_x11_screen_get_xscreen(gtk_widget_get_screen(dialog)));
 
         /* To prevent the settings dialog to be saved in the session */
         gdk_x11_set_sm_client_id("FAKE ID");
@@ -2158,10 +2201,10 @@ main(int argc, char **argv)
         gdk_notify_startup_complete();
 
         plug_child = GTK_WIDGET(gtk_builder_get_object(gxml, "alignment1"));
-        gtk_widget_reparent(plug_child, plug);
+        xfce_widget_reparent(plug_child, plug);
         gtk_widget_show(plug_child);
 
-        screen = gdk_screen_get_number(gtk_widget_get_screen(plug));
+        screen = XScreenNumberOfScreen(gdk_x11_screen_get_xscreen(gtk_widget_get_screen(plug)));
 
         /* In a GtkPlug setting there isn't an easy way to find our window
          * in cb_window_opened so we'll just force wnck to init and get the
