@@ -73,13 +73,15 @@ static void cb_xfdesktop_application_reload(GAction  *action,
                                             GVariant *parameter,
                                             gpointer  data);
 
+static void cb_xfdesktop_application_next(GAction  *action,
+                                          GVariant *parameter,
+                                          gpointer  data);
+
 static void xfdesktop_handle_quit_signals(gint sig, gpointer user_data);
 static void cb_xfdesktop_application_quit(GAction  *action,
                                           GVariant *parameter,
                                           gpointer  data);
 
-static gint xfdesktop_application_get_current_screen_number(XfdesktopApplication *app);
-
 static void cb_xfdesktop_application_menu(GAction  *action,
                                           GVariant *parameter,
                                           gpointer  data);
@@ -88,6 +90,10 @@ static void cb_xfdesktop_application_arrange(GAction  *action,
                                              GVariant *parameter,
                                              gpointer  data);
 
+static void cb_xfdesktop_application_debug(GAction  *action,
+                                           GVariant *parameter,
+                                           gpointer  data);
+
 static gboolean cb_wait_for_window_manager(gpointer data);
 static void     cb_wait_for_window_manager_destroyed(gpointer data);
 
@@ -117,13 +123,8 @@ struct _XfdesktopApplication
 {
     GApplication parent;
 
-#if !GLIB_CHECK_VERSION (2, 32, 0)
-    GSimpleActionGroup *actions;
-#endif
-
-    GtkWidget **desktops;
+    GtkWidget *desktop;
     XfconfChannel *channel;
-    gint nscreens;
     guint wait_for_wm_timeout_id;
     XfceSMClient *sm_client;
     GCancellable *cancel;
@@ -158,11 +159,7 @@ xfdesktop_application_class_init(XfdesktopApplicationClass *klass)
 static void
 xfdesktop_application_add_action(XfdesktopApplication *app, GAction *action)
 {
-#if GLIB_CHECK_VERSION (2, 32, 0)
     g_action_map_add_action(G_ACTION_MAP(app), action);
-#else
-    g_simple_action_group_insert(app->actions, action);
-#endif
 }
 
 static void
@@ -172,16 +169,18 @@ xfdesktop_application_init(XfdesktopApplication *app)
 
     app->cancel = g_cancellable_new();
 
-#if !GLIB_CHECK_VERSION (2, 32, 0)
-    app->actions = g_simple_action_group_new();
-#endif
-
     /* reload action */
     action = g_simple_action_new("reload", NULL);
     g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_reload), app);
     xfdesktop_application_add_action(app, G_ACTION(action));
     g_object_unref(action);
 
+    /* next action */
+    action = g_simple_action_new("next", NULL);
+    g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_next), app);
+    xfdesktop_application_add_action(app, G_ACTION(action));
+    g_object_unref(action);
+
     /* quit action */
     action = g_simple_action_new("quit", NULL);
     g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_quit), app);
@@ -200,9 +199,11 @@ xfdesktop_application_init(XfdesktopApplication *app)
     xfdesktop_application_add_action(app, G_ACTION(action));
     g_object_unref(action);
 
-#if !GLIB_CHECK_VERSION (2, 32, 0)
-    g_application_set_action_group(G_APPLICATION(app), (GActionGroup*)app->actions);
-#endif
+    /* debug action, parameter toggles debug state */
+    action = g_simple_action_new("debug", G_VARIANT_TYPE_BOOLEAN);
+    g_signal_connect(action, "activate", G_CALLBACK(cb_xfdesktop_application_debug), app);
+    xfdesktop_application_add_action(app, G_ACTION(action));
+    g_object_unref(action);
 }
 
 static void
@@ -252,12 +253,8 @@ session_die(gpointer user_data)
 
     TRACE("entering");
 
-    /* If we somehow got here after the app has been released we don't need
-     * to do anything */
-    if(user_data == NULL || !XFDESKTOP_IS_APPLICATION(user_data))
-        return;
-
-    app = XFDESKTOP_APPLICATION(user_data);
+    /* Ensure we always have a valid reference so we can quit xfdesktop */
+    app = xfdesktop_application_get();
 
     /* Cancel the wait for wm check if it's still running */
     g_cancellable_cancel(app->cancel);
@@ -265,11 +262,7 @@ session_die(gpointer user_data)
     for(main_level = gtk_main_level(); main_level > 0; --main_level)
         gtk_main_quit();
 
-#if GLIB_CHECK_VERSION(2, 32, 0)
     g_application_quit(G_APPLICATION(app));
-#else
-    xfdesktop_application_shutdown(G_APPLICATION(app));
-#endif
 }
 
 static void
@@ -311,7 +304,7 @@ event_forward_to_rootwin(GdkScreen *gscreen, GdkEvent *event)
         xev2.button = xev.button;
     } else
         return;
-    xev.window = GDK_WINDOW_XWINDOW(gdk_screen_get_root_window(gscreen));
+    xev.window = GDK_WINDOW_XID(gdk_screen_get_root_window(gscreen));
     xev.root =  xev.window;
     xev.subwindow = None;
     xev.time = event->button.time;
@@ -349,22 +342,17 @@ static gboolean
 reload_idle_cb(gpointer data)
 {
     XfdesktopApplication *app = XFDESKTOP_APPLICATION(data);
-    gint i;
 
     TRACE("entering");
 
     /* If xfdesktop never started there's nothing to reload, xfdesktop will
      * now startup */
-    if(!app->desktops)
+    if(!app->desktop)
         return FALSE;
 
-    /* reload all the desktops */
-    for(i = 0; i < app->nscreens; ++i) {
-        if(app->desktops[i])
-            xfce_desktop_refresh(XFCE_DESKTOP(app->desktops[i]));
-    }
-
-    menu_reload();
+    /* reload all the desktop */
+    if(app->desktop)
+        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), FALSE);
 
     g_application_release(G_APPLICATION(app));
 
@@ -388,6 +376,28 @@ cb_xfdesktop_application_reload(GAction  *action,
     g_idle_add((GSourceFunc)reload_idle_cb, g_application);
 }
 
+static void
+cb_xfdesktop_application_next(GAction  *action,
+                              GVariant *parameter,
+                              gpointer  data)
+{
+    XfdesktopApplication *app;
+
+    TRACE("entering");
+
+    g_return_if_fail(XFDESKTOP_IS_APPLICATION(data));
+
+    app = XFDESKTOP_APPLICATION(data);
+
+    /* If xfdesktop never started there's nothing to do here */
+    if(!app->desktop)
+        return;
+
+    /* reload the desktop forcing the wallpaper to advance */
+    if(app->desktop)
+        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), TRUE);
+}
+
 static void
 xfdesktop_handle_quit_signals(gint sig,
                               gpointer user_data)
@@ -422,23 +432,6 @@ cb_xfdesktop_application_quit(GAction  *action,
     session_die(app);
 }
 
-static gint
-xfdesktop_application_get_current_screen_number(XfdesktopApplication *app)
-{
-    GdkDisplay *display = gdk_display_get_default();
-    GdkScreen *screen;
-    gint screen_num;
-
-    gdk_display_get_pointer(display, &screen, NULL, NULL, NULL);
-
-    screen_num = gdk_screen_get_number(screen);
-
-    if(screen_num >= app->nscreens) {
-        return -1;
-    }
-
-    return screen_num;
-}
 
 /* parameter is a boolean that determines whether to popup the primay or
  * windowlist menu */
@@ -449,7 +442,6 @@ cb_xfdesktop_application_menu(GAction  *action,
 {
     XfdesktopApplication *app;
     gboolean popup_root_menu;
-    gint screen_num;
 
     TRACE("entering");
 
@@ -463,16 +455,11 @@ cb_xfdesktop_application_menu(GAction  *action,
     popup_root_menu = g_variant_get_boolean(parameter);
     app = XFDESKTOP_APPLICATION(data);
 
-    screen_num = xfdesktop_application_get_current_screen_number(app);
-    /* not initialized */
-    if(screen_num < 0)
-        return;
-
     if(popup_root_menu) {
-        xfce_desktop_popup_root_menu(XFCE_DESKTOP(app->desktops[screen_num]),
+        xfce_desktop_popup_root_menu(XFCE_DESKTOP(app->desktop),
                                      0, GDK_CURRENT_TIME);
     } else {
-        xfce_desktop_popup_secondary_root_menu(XFCE_DESKTOP(app->desktops[screen_num]),
+        xfce_desktop_popup_secondary_root_menu(XFCE_DESKTOP(app->desktop),
                                                0, GDK_CURRENT_TIME);
     }
 }
@@ -483,7 +470,6 @@ cb_xfdesktop_application_arrange(GAction  *action,
                                  gpointer  data)
 {
     XfdesktopApplication *app;
-    gint screen_num;
 
     TRACE("entering");
 
@@ -493,12 +479,27 @@ cb_xfdesktop_application_arrange(GAction  *action,
 
     app = XFDESKTOP_APPLICATION(data);
 
-    screen_num = xfdesktop_application_get_current_screen_number(app);
-    /* not initialized */
-    if(screen_num < 0)
+    xfce_desktop_arrange_icons(XFCE_DESKTOP(app->desktop));
+}
+
+/* parameter is a boolean that determines whether to enable or disable the
+ * debug messages */
+static void
+cb_xfdesktop_application_debug(GAction  *action,
+                               GVariant *parameter,
+                               gpointer  data)
+{
+    TRACE("entering");
+
+    /* sanity checks */
+    if(!data || !XFDESKTOP_IS_APPLICATION(data))
         return;
 
-    xfce_desktop_arrange_icons(XFCE_DESKTOP(app->desktops[screen_num]));
+    if(!g_variant_is_of_type(parameter, G_VARIANT_TYPE_BOOLEAN))
+        return;
+
+    /* Toggle the debug state */
+    xfdesktop_debug_set(g_variant_get_boolean(parameter));
 }
 
 /* Cleans up the associated wait for wm resources */
@@ -527,7 +528,7 @@ cb_wait_for_window_manager(gpointer data)
 
     for(i = 0; i < wfwm->atom_count; i++) {
         if(XGetSelectionOwner(wfwm->dpy, wfwm->atoms[i]) == None) {
-            DBG("window manager not ready on screen %d", i);
+            XF_DEBUG("window manager not ready on screen %d", i);
             have_wm = FALSE;
             break;
         }
@@ -561,7 +562,7 @@ cb_wait_for_window_manager_destroyed(gpointer data)
         g_printerr("No window manager registered on screen 0. "
                    "To start the xfdesktop without this check, run with --disable-wm-check.\n");
     } else {
-        DBG("found window manager after %d tries", wfwm->counter);
+        XF_DEBUG("found window manager after %d tries", wfwm->counter);
     }
 
     /* start loading the desktop, hopefully a window manager is found, but it
@@ -625,7 +626,6 @@ xfdesktop_application_start(XfdesktopApplication *app)
 {
     GdkDisplay *gdpy;
     GError *error = NULL;
-    gint i;
     gchar buf[1024];
 
     TRACE("entering");
@@ -653,37 +653,32 @@ xfdesktop_application_start(XfdesktopApplication *app)
     if(!xfconf_init(&error)) {
         g_warning("%s: unable to connect to settings daemon: %s.  Defaults will be used",
                   PACKAGE, error->message);
-        g_error_free(error);
+        g_clear_error(&error);
         error = NULL;
     } else
         app->channel = xfconf_channel_get(XFDESKTOP_CHANNEL);
 
-    /* create an XfceDesktop for every screen */
-    app->nscreens = gdk_display_get_n_screens(gdpy);
-    app->desktops = g_new0(GtkWidget *, app->nscreens);
+    g_snprintf(buf, sizeof(buf), "/backdrop/screen%d/", 0);
+    app->desktop = xfce_desktop_new(gdk_display_get_default_screen(gdpy),
+                                    app->channel, buf);
 
-    for(i = 0; i < app->nscreens; i++) {
-        g_snprintf(buf, sizeof(buf), "/backdrop/screen%d/", i);
-        app->desktops[i] = xfce_desktop_new(gdk_display_get_screen(gdpy, i),
-                                            app->channel, buf);
+    /* hook into the scroll event so we can forward it to the window
+     * manager */
+    gtk_widget_add_events(app->desktop, GDK_BUTTON_PRESS_MASK
+                          | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK);
+    g_signal_connect(G_OBJECT(app->desktop), "scroll-event",
+                     G_CALLBACK(scroll_cb), app);
 
-        /* hook into the scroll event so we can forward it to the window
-         * manager */
-        gtk_widget_add_events(app->desktops[i], GDK_BUTTON_PRESS_MASK
-                              | GDK_BUTTON_RELEASE_MASK | GDK_SCROLL_MASK);
-        g_signal_connect(G_OBJECT(app->desktops[i]), "scroll-event",
-                         G_CALLBACK(scroll_cb), app);
+    menu_attach(XFCE_DESKTOP(app->desktop));
+    windowlist_attach(XFCE_DESKTOP(app->desktop));
 
-        menu_attach(XFCE_DESKTOP(app->desktops[i]));
-        windowlist_attach(XFCE_DESKTOP(app->desktops[i]));
+    /* display the desktop and try to put it at the bottom */
+    gtk_widget_realize(app->desktop);
+    gdk_window_lower(gtk_widget_get_window(app->desktop));
 
-        /* display the desktop and try to put it at the bottom */
-        gtk_widget_realize(app->desktops[i]);
-        gdk_window_lower(gtk_widget_get_window(app->desktops[i]));
+    xfce_desktop_set_session_logout_func(XFCE_DESKTOP(app->desktop),
+                                         session_logout);
 
-        xfce_desktop_set_session_logout_func(XFCE_DESKTOP(app->desktops[i]),
-                                             session_logout);
-    }
 
     menu_init(app->channel);
     windowlist_init(app->channel);
@@ -701,7 +696,7 @@ xfdesktop_application_start(XfdesktopApplication *app)
                                               app, NULL);
     } else {
         g_warning("Unable to set up POSIX signal handlers: %s", error->message);
-        g_error_free(error);
+        g_clear_error(&error);
     }
 
     gtk_main();
@@ -720,7 +715,6 @@ static void
 xfdesktop_application_shutdown(GApplication *g_application)
 {
     XfdesktopApplication *app = XFDESKTOP_APPLICATION(g_application);
-    gint i;
 
     TRACE("entering");
 
@@ -732,15 +726,12 @@ xfdesktop_application_shutdown(GApplication *g_application)
     menu_cleanup();
     windowlist_cleanup();
 
-    if(app->desktops) {
-        for(i = 0; i < app->nscreens; i++) {
-            /* ensure the desktops get freed if they haven't been */
-            if(app->desktops[i] && GTK_IS_WIDGET(app->desktops[i]))
-                gtk_widget_destroy(app->desktops[i]);
-        }
+    if(app->desktop) {
+        /* ensure the desktop gets freed if it hasen't been */
+        if(app->desktop && GTK_IS_WIDGET(app->desktop))
+            gtk_widget_destroy(app->desktop);
 
-        g_free(app->desktops);
-        app->desktops = NULL;
+        app->desktop = NULL;
     }
 
     xfconf_shutdown();
@@ -763,19 +754,25 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     GError *error = NULL;
     gboolean opt_version = FALSE;
     gboolean opt_reload = FALSE;
+    gboolean opt_next = FALSE;
     gboolean opt_menu = FALSE;
     gboolean opt_windowlist = FALSE;
     gboolean opt_arrange = FALSE;
     gboolean opt_quit = FALSE;
+    gboolean opt_enable_debug = FALSE;
+    gboolean opt_disable_debug = FALSE;
     gboolean option_set = FALSE;
     const GOptionEntry main_entries[] = {
         { "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version, N_("Display version information"), NULL },
         { "reload", 'R', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_reload, N_("Reload all settings"), NULL },
+        { "next", 'N', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_next, N_("Advance to the next wallpaper on the current workspace"), NULL },
         { "menu", 'M', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_menu, N_("Pop up the menu (at the current mouse position)"), NULL },
         { "windowlist", 'W', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_windowlist, N_("Pop up the window list (at the current mouse position)"), NULL },
 #ifdef ENABLE_FILE_ICONS
         { "arrange", 'A', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_arrange, N_("Automatically arrange all the icons on the desktop"), NULL },
 #endif
+        { "enable-debug", 'e', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_enable_debug, N_("Enable debug messages"), NULL },
+        { "disable-debug", 'd', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_disable_debug, N_("Disable debug messages"), NULL },
         { "disable-wm-check", 'D', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &app->opt_disable_wm_check, N_("Do not wait for a window manager on startup"), NULL },
         { "quit", 'Q', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_quit, N_("Cause xfdesktop to quit"), NULL },
         { NULL, 0, 0, 0, NULL, NULL, NULL }
@@ -794,7 +791,7 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     if(!g_option_context_parse(octx, &argc, arguments, &error)) {
         g_printerr(_("Failed to parse arguments: %s\n"), error->message);
         g_option_context_free(octx);
-        g_error_free(error);
+        g_clear_error(&error);
         *exit_status = 1;
         return TRUE;
     }
@@ -846,6 +843,9 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     } else if(opt_reload) {
         g_action_group_activate_action(G_ACTION_GROUP(g_application), "reload", NULL);
         option_set = TRUE;
+    } else if(opt_next) {
+        g_action_group_activate_action(G_ACTION_GROUP(g_application), "next", NULL);
+        option_set = TRUE;
     } else if(opt_menu) {
         g_action_group_activate_action(G_ACTION_GROUP(g_application), "menu",
                                        g_variant_new_boolean(TRUE));
@@ -857,6 +857,14 @@ xfdesktop_application_local_command_line(GApplication *g_application,
     } else if(opt_arrange) {
         g_action_group_activate_action(G_ACTION_GROUP(g_application), "arrange", NULL);
         option_set = TRUE;
+    } else if(opt_enable_debug) {
+        g_action_group_activate_action(G_ACTION_GROUP(g_application), "debug",
+                                       g_variant_new_boolean(TRUE));
+        option_set = TRUE;
+    } else if(opt_disable_debug) {
+        g_action_group_activate_action(G_ACTION_GROUP(g_application), "debug",
+                                       g_variant_new_boolean(FALSE));
+        option_set = TRUE;
     }
 
     /* We handled the command line option */
