@@ -22,7 +22,7 @@
 #ifndef __XFDESKTOP_FILE_UTILS_H__
 #define __XFDESKTOP_FILE_UTILS_H__
 
-#include <dbus/dbus-glib.h>
+#include <gio/gio.h>
 
 #ifdef HAVE_THUNARX
 #include <thunarx/thunarx.h>
@@ -39,9 +39,6 @@ GKeyFile *xfdesktop_file_utils_query_key_file(GFile *file,
 gchar *xfdesktop_file_utils_get_display_name(GFile *file,
                                              GFileInfo *info);
 
-gboolean xfdesktop_file_utils_volume_is_present(GVolume *volume);
-gboolean xfdesktop_file_utils_volume_is_removable(GVolume *volume);
-
 GList *xfdesktop_file_utils_file_icon_list_to_file_list(GList *icon_list);
 GList *xfdesktop_file_utils_file_list_from_string(const gchar *string);
 gchar *xfdesktop_file_utils_file_list_to_string(GList *file_list);
@@ -70,6 +67,10 @@ void xfdesktop_file_utils_open_folder(GFile *file,
 void xfdesktop_file_utils_rename_file(GFile *file,
                                       GdkScreen *screen,
                                       GtkWindow *parent);
+void xfdesktop_file_utils_bulk_rename(GFile *working_directory,
+                                      GList *files,
+                                      GdkScreen *screen,
+                                      GtkWindow *parent);
 void xfdesktop_file_utils_trash_files(GList *files,
                                        GdkScreen *screen,
                                        GtkWindow *parent);
@@ -112,8 +113,6 @@ gboolean xfdesktop_file_utils_transfer_files(GdkDragAction action,
 
 
 gboolean xfdesktop_file_utils_dbus_init(void);
-DBusGProxy *xfdesktop_file_utils_peek_trash_proxy(void);
-DBusGProxy *xfdesktop_file_utils_peek_filemanager_proxy(void);
 void xfdesktop_file_utils_dbus_cleanup(void);
 
 
