@@ -27,7 +27,6 @@
 #endif
 
 #include <libwnck/libwnck.h>
-#include <libwnck/window-action-menu.h>
 #include <libxfce4ui/libxfce4ui.h>
 
 #include "xfdesktop-window-icon.h"
@@ -43,7 +42,7 @@ static void xfdesktop_window_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
                                                    gint width, gint height);
-static G_CONST_RETURN gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
+static const gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
 static gchar *xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon);
 
 static gboolean xfdesktop_window_icon_activated(XfdesktopIcon *icon);
@@ -88,7 +87,7 @@ xfdesktop_window_icon_finalize(GObject *obj)
 {
     XfdesktopWindowIcon *icon = XFDESKTOP_WINDOW_ICON(obj);
     gchar data_name[256];
-    guint16 row, col;
+    gint16 row, col;
     
     g_free(icon->priv->label);
     
@@ -156,7 +155,7 @@ xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
     return pix;
 }
 
-static G_CONST_RETURN gchar *
+static const gchar *
 xfdesktop_window_icon_peek_label(XfdesktopIcon *icon)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
@@ -189,7 +188,7 @@ xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
                                             GtkWidget *menu)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    GtkWidget *amenu = wnck_create_window_action_menu(window_icon->priv->window);
+    GtkWidget *amenu = wnck_action_menu_new(window_icon->priv->window);
     GtkWidget *mi;
 
     mi = gtk_menu_item_new_with_mnemonic(_("_Window Actions"));
